class Time

#------------------------------------------------------------
def Time.days_per_month(month)

	days_per_month = {
		:jan => 31,:feb => 28,:mar => 31,:apr => 30,
		:may => 31,:jun => 30,:jul => 31,:aug => 31,
		:sep => 30,:oct => 31,:nov => 30,:dec => 31 }

	if month.is_a?(Integer)
		the_month = [:jan,:feb,:mar,:apr,:may,:jun,:jul,:aug,:sep,:nov,:dec][month-1]
	elsif month.is_a?(Symbol)
		the_month = month.to_s[0..2].downcase.to_sym
	elsif month.is_a?(String)
		if month.scan(/\d{1,2}/)
			the_month = [:jan,:feb,:mar,:apr,:may,:jun,:jul,:aug,:sep,:nov,:dec][month.to_i - 1]
		else
			the_month = month[0..2].downcase.to_sym
		end
	end

	days_per_month[the_month]

end

#------------------------------------------------------------
def Time.convert_month_name(this_name)

	this_name.downcase!

	# plugin for languages some day
	month_names = [
		['01', ['jan', 'january']],
		['02', ['feb', 'february']],
		['03', ['mar', 'march']],
		['04', ['apr', 'april']],
		['05', ['may']],
		['06', ['jun', 'june']],
		['07', ['jul', 'july']],
		['08', ['aug', 'august']],
		['09', ['sep', 'september']],
		['10', ['oct', 'october']],
		['11', ['nov', 'november']],
		['12', ['dec', 'december']]  ]

	month_names.each do |this_month|
		month_number = this_month[0]
		month_names = this_month[1]
		if month_names.include?(this_name)
			return month_number
		end
	end
end

#------------------------------------------------------------
def Time.from_string(date_str, mode=:usa)

# mode options = :usa, :euro

# Dec 25 2001
# 25 Dec 2001
# 2001 Dec 25
# 2001 25 Dec
# Dec 12 01
	
# Dec 25
# 25 Dec
	
# 2001-25-12
# 2001/12/25
# 2001 12, 25
# 25-12-2001
# 25-12-01
# 25-12-99
# 12-25-01
# 12-25-2001

	if date_str == 'now' || date_str == 'Now'
		return Time.now
	end

	if date_str == 'today' || date_str == 'Today'
		return Time.now.add_hours(8)
	end

	if date_str == 'tomorrow' || date_str == 'Tomorrow'
		return Time.now.add_days(1).add_hours(8)
	end

	the_year	= nil
	the_month	= nil
	the_day		= nil
	date_parts 	= date_str.gsub(/[-\/\. ,:]+/, '-').split('-')

	if date_parts.size == 2
		the_year = Time.new.year
		date_parts << the_year.to_s
		part_c = the_year.to_s
	end

	if date_parts.size != 3
		return nil
	end

	part_a = date_parts[0].dup
	part_b = date_parts[1].dup
	part_c = date_parts[2].dup

	#-----------------------------------
	# parse the DATE

	if    !part_a.scan(/\d{4}/)[0].nil? 	then the_year = part_a
	elsif !part_b.scan(/\d{4}/)[0].nil?		then the_year = part_b
	elsif !part_c.scan(/\d{4}/)[0].nil?		then the_year = part_c
	else; the_year = part_c
	end

	if the_year.length == 2
		if the_year.to_i < 40
			the_year = '20' + the_year
			part_c = the_year
		else
			the_year = '19' + the_year
			part_c = the_year
		end
	end

	if !part_a.scan(/[a-zA-Z]/).empty?
		the_month 		= convert_month_name(part_a)
		the_day 		= part_b
	elsif !part_b.scan(/[a-zA-Z]/).empty?
		the_month 		= convert_month_name(part_b)
		part_c == the_year 	? the_day = part_a : the_day = part_c
	elsif !part_c.scan(/[a-zA-Z]/).empty?
		the_month 		= convert_month_name(part_c)
		the_day 		= part_b
	end

	if the_month.nil?
		date_parts.delete(the_year)
		if (mode == :euro) || (date_parts[0].to_i > 12)
			the_day = date_parts[0]
			the_month = date_parts[1]
		else
			the_day = date_parts[1]
			the_month = date_parts[0]
		end
	end
	
	
	#-----------------------------------
	# parse the TIME
	
	#::ToDo::

	#-----------------------------------
	# format the Time object

	the_hour = 8
	
	begin
		the_date = Time.local(the_year, the_month, the_day, the_hour)
	rescue
		the_date = nil
	end
	
	return the_date

end

#: -
#------------------------------------------------------------
def add_years(raw_units)
	time_parts 	= self.to_a
	the_year	= self.year.to_i
	new_year 	= the_year + raw_units

	time_parts[5] = new_year

	return Time.local(time_parts[0], 
		time_parts[1], time_parts[2], time_parts[3],
		time_parts[4], time_parts[5], time_parts[6],
		time_parts[7], time_parts[8], time_parts[9])
end

#------------------------------------------------------------
def add_months(raw_units)

	time_parts 	= self.to_a
	the_year	= self.year.to_i
	the_month	= self.month.to_i
	the_day		= self.day.to_i

	total_months = raw_units + the_month

	if total_months == 12
		new_month = 12
		new_year = the_year
	else
		new_month = total_months%12
		new_month = the_month if new_month == 0
		new_year = the_year + (total_months/12)
	end

	new_day = the_day

	if the_month == 2 && the_day > 28
		new_day = 1
		new_month += 1
	elsif [4,6,9,11].include?(self.month) && the_day == 30
		new_day = 1
		new_month += 1
	end

	if new_month > 12
		new_year += 1 
		new_month = 1
	end

	time_parts[3] = new_day
	time_parts[4] = new_month
	time_parts[5] = new_year

	return Time.local(time_parts[0], 
		time_parts[1], time_parts[2], time_parts[3],
		time_parts[4], time_parts[5], time_parts[6],
		time_parts[7], time_parts[8], time_parts[9])

end

#------------------------------------------------------------
def add_quarters(raw_units)
	self.add_months(raw_units * 3)
end

#------------------------------------------------------------
def add_weeks(raw_units)
	total_seconds = raw_units.to_i * 7 * 24 * 60 * 60
	self + total_seconds
end

#------------------------------------------------------------
def add_days(raw_units)
	total_seconds = raw_units.to_i * 24 * 60 * 60
	self + total_seconds
end

#------------------------------------------------------------
def add_hours(raw_units)
	total_seconds = raw_units.to_i * 60 * 60
	self + total_seconds
end

#------------------------------------------------------------
def add_minutes(raw_units)
	total_seconds = raw_units.to_i * 60
	self + total_seconds
end

#------------------------------------------------------------
def add_seconds(raw_units)
	total_seconds = raw_units.to_i
	self + total_seconds
end

#: -
#------------------------------------------------------------
def subtract_years(raw_units)
	time_parts 	= self.to_a
	the_year	= self.year.to_i
	new_year 	= the_year - raw_units

	time_parts[5] = new_year

	return Time.local(time_parts[0], 
		time_parts[1], time_parts[2], time_parts[3],
		time_parts[4], time_parts[5], time_parts[6],
		time_parts[7], time_parts[8], time_parts[9])
end

#------------------------------------------------------------
def subtract_months(raw_units)

	time_parts 	= self.to_a
	the_year	= self.year.to_i
	the_month	= self.month.to_i
	the_day		= self.day.to_i

	total_months = the_month - raw_units

	if total_months == 0
		new_month = 1
		new_year = the_year
	else
		new_month = total_months%12
		new_year = the_year + (total_months/12)
	end

	new_day = the_day

	if the_month == 2 && the_day > 28
		new_day = 1
		new_month += 1
	elsif [4,6,9,11].include?(self.month) && the_day == 30
		new_day = 1
		new_month += 1
	end

	time_parts[3] = new_day
	time_parts[4] = new_month
	time_parts[5] = new_year

	return Time.local(time_parts[0], 
		time_parts[1], time_parts[2], time_parts[3],
		time_parts[4], time_parts[5], time_parts[6],
		time_parts[7], time_parts[8], time_parts[9])

end

#------------------------------------------------------------
def subtract_quarters(raw_units)
	self.subtract_months(raw_units * 3)
end

#------------------------------------------------------------
def subtract_weeks(raw_units)
	total_seconds = raw_units.to_i * 7 * 24 * 60 * 60
	self - total_seconds
end

#------------------------------------------------------------
def subtract_days(raw_units)
	total_seconds = raw_units.to_i * 24 * 60 * 60
	self - total_seconds
end

#------------------------------------------------------------
def subtract_hours(raw_units)
	total_seconds = raw_units.to_i * 60 * 60
	self - total_seconds
end

#------------------------------------------------------------
def subtract_minutes(raw_units)
	total_seconds = raw_units.to_i * 60
	self - total_seconds
end

#------------------------------------------------------------
def subtract_seconds(raw_units)
	total_seconds = raw_units.to_i
	self - total_seconds
end

#: -
#------------------------------------------------------------
def as_iso_date_time
	return self.strftime("%Y-%m-%d %H:%M:%S")
end

#------------------------------------------------------------
def as_iso_date
	return self.strftime("%Y-%m-%d")
end

#------------------------------------------------------------
def as_moDDYYYY
	return self.strftime("%b %d, %Y")
end

#------------------------------------------------------------
def as_moDDYYYY_time12Mins
	return self.strftime("%b %d, %Y %I:%M %p")
end

#------------------------------------------------------------
def as_moDDYYYY_time24Secs
	return self.strftime("%b %d, %Y %H:%M:%S")
end

#------------------------------------------------------------
def as_monthDDYYYY
	return self.strftime("%B %d, %Y")
end

#------------------------------------------------------------
def as_moYYYY
	return self.strftime("%b %Y")
end

#------------------------------------------------------------
def as_monthYYYY
	return self.strftime("%B %Y")
end

#: -
#------------------------------------------------------------
def time12_with_seconds
	return self.strftime("%I:%M:%S %p")
end

#------------------------------------------------------------
def time12_with_minutes
	return self.strftime("%I:%M %p")
end

#------------------------------------------------------------
def time24_with_seconds
	return self.strftime("%H:%M:%S")
end

#------------------------------------------------------------
def time24_with_minutes
	return self.strftime("%H:%M")
end

#: -
#------------------------------------------------------------
def days_elapsed
	unless self.nil?
		return (Time.now - self) / 86400
	end
	return 0
end

#------------------------------------------------------------
def hours_elapsed
	unless self.nil?
		return (Time.now - self) / 3600
	end
	return 0
end

#------------------------------------------------------------
def secs_elapsed
	unless self.nil?
		return (Time.now - self)
	end
	return 0
end

#------------------------------------------------------------
def mins_elapsed
	unless self.nil?
		return (Time.now - self) / 60
	end
	return 0
end


end
