module ExtensionsTimeSince

#------------------------------------------------------------
def normalize_to_times(origin)

	point_in_time = self.to_time
	case origin.class.to_s
		when 'String'
			origin = Time.from_string(origin)
		when 'Date'
			origin = origin.to_time
		when 'Time'
		else
			origin = nil
	end
	
	return {:point_in_time => point_in_time, :origin => origin}
end

#------------------------------------------------------------
def as_age_from(origin)
	as_years_since(origin)
end

#------------------------------------------------------------
def as_age
	point_in_time = Time.now
	origin = self.to_time
	unless self.nil?
		return ((point_in_time - origin) / 31536000).floor
	end
	return ''
end

#: -
#------------------------------------------------------------
# The since times are based on "how much time has passed to 'me' (self) 
# since this other time 'origin'" which assumes that self is some point 
# in time considered to be now or the end, being compared to a reference 
# point considered the origin or the start if now is before the origin 
# it will be a negative number. If no origin is passed, these methods 
# end up working just like the elapsed methods.

#------------------------------------------------------------
def as_years_since(origin=Time.now)
	times = normalize_to_times(origin)
	unless times[:point_in_time].nil? || times[:origin].nil?
		return ((times[:point_in_time] - times[:origin] ) / 31536000).floor
	end
	return ''
end

#------------------------------------------------------------
def as_months_since(origin=Time.now)
	times = normalize_to_times(origin)
	unless times[:point_in_time].nil? || times[:origin].nil?
		return ((times[:point_in_time] - times[:origin] ) / (31536000/12)).floor
	end
	return ''
end

#------------------------------------------------------------
def as_weeks_since(origin=Time.now)
	times = normalize_to_times(origin)
	unless times[:point_in_time].nil? || times[:origin].nil?
		return ((times[:point_in_time] - times[:origin] ) / 6048000).floor
	end
	return ''
end

#------------------------------------------------------------
def as_days_since(origin=Time.now)
	times = normalize_to_times(origin)
	unless times[:point_in_time].nil? || times[:origin].nil?
		return ((times[:point_in_time] - times[:origin] ) / 86400).floor
	end
	return ''
end

#------------------------------------------------------------
def as_hours_since(origin=Time.now)
	times = normalize_to_times(origin)
	unless times[:point_in_time].nil? || times[:origin].nil?
		return ((times[:point_in_time] - times[:origin] ) / 3600).floor
	end
	return ''
end

#------------------------------------------------------------
def as_secs_since(origin=Time.now)
	times = normalize_to_times(origin)
	unless times[:point_in_time].nil? || times[:origin].nil?
		return (times[:point_in_time] - times[:origin]).floor
	end
	return ''
end

#------------------------------------------------------------
def as_mins_since(origin=Time.now)
	times = normalize_to_times(origin)
	unless times[:point_in_time].nil? || times[:origin].nil?
		return ((times[:point_in_time] - times[:origin] ) / 60).floor
	end
	return ''
end

#: -
#------------------------------------------------------------
def days_elapsed
	unless self.nil?
		return (Time.now - self) / 86400
	end
	return 0
end

#------------------------------------------------------------
def hours_elapsed
	unless self.nil?
		return (Time.now - self) / 3600
	end
	return 0
end

#------------------------------------------------------------
def secs_elapsed
	unless self.nil?
		return (Time.now - self)
	end
	return 0
end

#------------------------------------------------------------
def mins_elapsed
	unless self.nil?
		return (Time.now - self) / 60
	end
	return 0
end

end
